@@ -137,6 +137,8 @@ struct msc_a {
 		/* There may be up to 7 incoming calls for this subscriber. This is the currently serviced voice call,
 		 * as in, the other person the subscriber is currently talking to. */
 		struct gsm_trans *active_trans;
+
+		struct osmo_timer_list assignment_request_pending;
 	} cc;
 
 	struct msc_ho_state ho;
@@ -144,6 +146,9 @@ struct msc_a {
 	struct osmo_use_count use_count;
 	struct osmo_use_count_entry use_count_buf[8];
 	int32_t max_total_use_count;
+
+	/* Used for scheduling X36 (additional delay for LU connections) */
+	struct osmo_timer_list lu_delay_timer;
 };
 
 osmo_static_assert(offsetof(struct msc_a, c) == 0, msc_role_common_first_member_of_msc_a);
@@ -216,6 +221,7 @@ bool msc_a_is_establishing_auth_ciph(const struct msc_a *msc_a);
 
 int msc_a_ensure_cn_local_rtp(struct msc_a *msc_a, struct gsm_trans *cc_trans);
 int msc_a_try_call_assignment(struct gsm_trans *cc_trans);
+void msc_a_tx_assignment_cmd(struct msc_a *msc_a);
 
 const char *msc_a_cm_service_type_to_use(struct msc_a *msc_a, enum osmo_cm_service_type cm_service_type);
 
