@@ -25,7 +25,7 @@ struct sdp_audio_codec {
 	/* Like "GSM", "AMR", "EFR", ... */
 	char subtype_name[16];
 	unsigned int rate;
-	char fmtp[64];
+	char fmtp[256];
 };
 
 struct sdp_audio_codecs {
@@ -41,8 +41,8 @@ struct sdp_msg {
 	struct csd_bs_list bearer_services;
 };
 
-#define foreach_sdp_audio_codec(/* struct sdp_audio_codec* */ CODEC, \
-				/* struct sdp_audio_codecs* */ AC) \
+#define sdp_audio_codecs_foreach(/* struct sdp_audio_codec* */ CODEC, \
+				 /* struct sdp_audio_codecs* */ AC) \
 	for (CODEC = (AC)->codec; \
 	     (CODEC - (AC)->codec) < OSMO_MIN((AC)->count, ARRAY_SIZE((AC)->codec)); \
 	     CODEC++)
