@@ -763,6 +763,8 @@ static int config_write_msc(struct vty *vty)
 	vty_out(vty, "msc%s", VTY_NEWLINE);
 	if (gsmnet->mncc_sock_path)
 		vty_out(vty, " mncc external %s%s", gsmnet->mncc_sock_path, VTY_NEWLINE);
+	else
+		vty_out(vty, " mncc internal%s", VTY_NEWLINE);
 	vty_out(vty, " mncc guard-timeout %i%s",
 		gsmnet->mncc_guard_timeout, VTY_NEWLINE);
 	vty_out(vty, " ncss guard-timeout %i%s",
@@ -1991,7 +1993,7 @@ DEFUN(cfg_hlr_ipa_name,
       "Set the IPA name of this MSC\n"
       "A unique name for this MSC. For example: PLMN + redundancy server number: MSC-901-70-0. "
       "This name is used for GSUP routing and must be set if more than one MSC is connected to the HLR. "
-      "The default is 'MSC-00-00-00-00-00-00'.\n")
+      "The default is 'unnamed-MSC'.\n")
 {
 	if (vty->type != VTY_FILE) {
 		vty_out(vty, "The IPA name cannot be changed at run-time; "
