@@ -23,7 +23,7 @@
 #include <osmocom/core/fsm.h>
 #include <osmocom/core/utils.h>
 #include <osmocom/gsm/gsup.h>
-#include <osmocom/vlr/vlr.h>
+#include <osmocom/msc/vlr.h>
 #include <osmocom/msc/debug.h>
 
 #include "vlr_core.h"
@@ -346,7 +346,7 @@ static void auth_fsm_needs_auth(struct osmo_fsm_inst *fi, uint32_t event, void *
 	} else {
 		/* go straight ahead with sending auth request */
 		osmo_fsm_inst_state_chg(fi, VLR_SUB_AS_WAIT_RESP,
-					vlr_timer_secs(vsub->vlr, 3260), 3260);
+					vlr_timer(vsub->vlr, 3260), 3260);
 		_vlr_subscr_authenticate(fi);
 	}
 }
@@ -358,6 +358,7 @@ static void auth_fsm_wait_ai(struct osmo_fsm_inst *fi, uint32_t event,
 	struct auth_fsm_priv *afp = fi->priv;
 	struct vlr_subscr *vsub = afp->vsub;
 	struct osmo_gsup_message *gsup = data;
+	enum gsm48_reject_value gsm48_rej;
 
 	if (event == VLR_AUTH_E_HLR_SAI_NACK)
 		LOGPFSM(fi, "GSUP: rx Auth Info Error cause: %d: %s\n",
@@ -390,17 +391,19 @@ static void auth_fsm_wait_ai(struct osmo_fsm_inst *fi, uint32_t event,
 	case VLR_AUTH_E_HLR_SAI_NACK:
 		/* HLR did not return Auth Info, hence cannot authenticate. (The caller may still decide to permit
 		 * attaching without authentication) */
-		auth_fsm_term(fi, AUTH_FSM_NO_AUTH_INFO, vlr_gmm_cause_to_reject_cause_domain(gsup->cause, true));
+		vlr_gmm_cause_to_mm_cause(gsup->cause, &gsm48_rej);
+		auth_fsm_term(fi, AUTH_FSM_NO_AUTH_INFO, gsm48_rej);
 		break;
 	case VLR_AUTH_E_HLR_SAI_ABORT:
-		auth_fsm_term(fi, AUTH_FSM_FAILURE, vlr_gmm_cause_to_reject_cause_domain(gsup->cause, true));
+		vlr_gmm_cause_to_mm_cause(gsup->cause, &gsm48_rej);
+		auth_fsm_term(fi, AUTH_FSM_FAILURE, gsm48_rej);
 		break;
 	}
 
 	return;
 pass:
 	osmo_fsm_inst_state_chg(fi, VLR_SUB_AS_WAIT_RESP,
-				vlr_timer_secs(vsub->vlr, 3260), 3260);
+				vlr_timer(vsub->vlr, 3260), 3260);
 	_vlr_subscr_authenticate(fi);
 }
 
@@ -424,7 +427,7 @@ static void auth_fsm_wait_auth_resp(struct osmo_fsm_inst *fi, uint32_t event,
 						   GSM_MI_TYPE_IMSI);
 				osmo_fsm_inst_state_chg(fi,
 						VLR_SUB_AS_WAIT_ID_IMSI,
-						vlr_timer_secs(vlr, 3270), 3270);
+						vlr_timer(vlr, 3270), 3270);
 			} else {
 				auth_fsm_term(fi, AUTH_FSM_FAILURE, GSM48_REJECT_ILLEGAL_MS);
 			}
@@ -467,13 +470,13 @@ static void auth_fsm_wait_ai_resync(struct osmo_fsm_inst *fi,
 	case VLR_AUTH_E_HLR_SAI_ACK:
 		vlr_subscr_update_tuples(vsub, gsup);
 		osmo_fsm_inst_state_chg(fi, VLR_SUB_AS_WAIT_RESP_RESYNC,
-					vlr_timer_secs(vsub->vlr, 3260), 3260);
+					vlr_timer(vsub->vlr, 3260), 3260);
 		_vlr_subscr_authenticate(fi);
 		break;
 	case VLR_AUTH_E_HLR_SAI_NACK:
 		auth_fsm_term(fi,
 			      AUTH_FSM_FAILURE,
-			      gsup->cause == GMM_CAUSE_IMSI_UNKNOWN ?
+			      gsup->cause == GMM_CAUSE_IMSI_UNKNOWN?
 				      GSM48_REJECT_IMSI_UNKNOWN_IN_HLR
 				      : GSM48_REJECT_NETWORK_FAILURE);
 		break;
@@ -500,7 +503,7 @@ static void auth_fsm_wait_auth_resp_resync(struct osmo_fsm_inst *fi,
 						   GSM_MI_TYPE_IMSI);
 				osmo_fsm_inst_state_chg(fi,
 						VLR_SUB_AS_WAIT_ID_IMSI,
-						vlr_timer_secs(vlr, 3270), 3270);
+						vlr_timer(vlr, 3270), 3270);
 			} else {
 				/* Result = Aborted */
 				auth_fsm_term(fi, AUTH_FSM_FAILURE, GSM48_REJECT_SYNCH_FAILURE);
