@@ -2,6 +2,8 @@
 
 #include <osmocom/core/utils.h>
 
+struct vlr_subscr;
+
 /* Parameters to VLR_AUTH_E_MS_AUTH_RESP */
 struct vlr_auth_resp_par {
 	bool is_r99;
@@ -27,12 +29,13 @@ enum vlr_fsm_auth_event {
 	VLR_AUTH_E_MS_ID_IMSI,
 };
 
-struct osmo_fsm vlr_auth_fsm;
+extern struct osmo_fsm vlr_auth_fsm;
 
 struct osmo_fsm_inst *auth_fsm_start(struct vlr_subscr *vsub,
-				     uint32_t log_level,
 				     struct osmo_fsm_inst *parent,
-				     uint32_t parent_term_event,
+				     uint32_t parent_event_success,
+				     uint32_t parent_event_no_auth_info,
+				     uint32_t parent_event_failure,
 				     bool is_r99,
 				     bool is_utran);
 
